@@ -4,7 +4,7 @@
 /* ------------------------------------------------------------------------- */
 
 /*
- * Copyright 2007-2014 GRAHAM DUMPLETON
+ * Copyright 2007-2015 GRAHAM DUMPLETON
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -31,11 +31,13 @@ extern int wsgi_dump_stack_traces;
 
 extern apr_thread_mutex_t* wsgi_monitor_lock;
 
-extern PyMethodDef wsgi_get_utilization_method[];
+extern PyMethodDef wsgi_process_metrics_method[];
 
 extern double wsgi_start_request(void);
 extern double wsgi_end_request(void);
 
+extern PyMethodDef wsgi_server_metrics_method[];
+
 /* ------------------------------------------------------------------------- */
 
 #endif
